#!/usr/bin/env python3
#-------------------------------------------------------#
# %%
import math
import numpy as np
import matplotlib.pyplot as plt
from scipy.interpolate import interp1d, splprep, splev
from shapely.geometry import Polygon, Point
from shapely import affinity
#-------------------------------------------------------#
# Turbine class definition
class TwinEM0:

    '''
        overW   overall width
    '''
    overW = 4.7752
    overL = 2.413
    overH = 2.12
    fndH = 0.15

    colL = 0.14
    colW = 0.12
    colH = 1.97

    wallThick = colW
    wallH = colH
    wallWDis = 2.33
    wallLDis = 1.13

    curveW = 0.13
    curvePt2WDis = 0.26     # Distance from a point on the curve to the closest column's corner
    curvePt2LDis = 0.63     # Distance from a point on the curve to the closest column's corner
    curvePt3WDis = 0.32     # Distance from the mid point on the curve to the closest column's corner
    curvePt5LDis = 0.60
    curvePt5WDis = 0.34
    
    res = 0.0625/2
    #overW -= res

    def __init__(self, loc, elev, slope, orientation):
        self.loc = loc
        self.elev = elev
        self.slope = slope
        self.orientation = orientation

    def getBathy(self, W, L, loc=[0.0,0.0]):
        x = np.arange(-W/2,W/2,self.res) + loc[0]
        y = np.arange(-L/2,L/2,self.res) + loc[1]
        xv,yv = np.meshgrid(x,y)
        return xv.flatten(),yv.flatten()

    def getBathyBase(self):
        xv,yv = self.getBathy(self.overW,self.overL,self.loc)
        return xv,yv

    def getBathyCols(self):
        xC = np.array([])
        yC = np.array([])
        cW = [-1,-1]
        cL = [-1,1]

        for i in range(2):
            xv,yv = self.getBathy(self.colW,self.colL,
                    [cW[i]*self.wallWDis,cL[i]*self.wallLDis])
           
            xC = np.append(xC,xv)
            yC = np.append(yC,yv)
            
        return xC.flatten(), yC.flatten()

    def getBathyFrame(self):
        # Inner and outer walls
        pt1 = [-(self.overW/2-self.colW),self.overL/2-self.colL]
        pt2 = [-(self.overW/2-self.colW-self.curvePt2WDis),self.overL/2-self.colL-self.curvePt2LDis]
        pt3 = [-(self.overW/2-self.colW-self.curvePt3WDis+self.res/2),self.res/2]
        pt4 = [pt1[0],self.overL/2]
        pt5 = [-(self.overW/2-self.colW-self.curvePt5WDis),self.overL/2-self.colL-self.curvePt5LDis]
        pt6 = [pt3[0]+self.curveW-self.res/2,pt3[1]]

        C1x = [pt1[0], pt2[0], pt3[0]]
        C1y = [pt1[1], pt2[1], pt3[1]]
        C2x = [pt4[0], pt5[0], pt6[0]]
        C2y = [pt4[1], pt5[1], pt6[1]]

        tck, u = splprep([C1x, C1y], s=0, k=2)
        xC1,yC1 = splev(np.arange(0, 1+self.res, self.res), tck)

        tck, u = splprep([C2x, C2y], s=0, k=2)
        xC2,yC2 = splev(np.arange(0, 1+self.res, self.res), tck)

        # Construct the polygon
        xC = np.append(xC1[0],xC2)
        yC = np.append(yC1[0],yC2)
        xC = np.append(xC,np.flip(xC2))
        yC = np.append(yC,-np.flip(yC2))
        xC = np.append(xC,xC1)
        yC = np.append(yC,-(yC1))
        xC = np.append(xC,np.flip(xC1))
        yC = np.append(yC,np.flip(yC1))

        # Create background grid
        x = np.arange(-self.overW/2, 0,self.res)
        y = np.arange(-self.overL/2, self.overL/2,self.res)
        xv,yv = np.meshgrid(x,y)

        # Subtract points outside the polygon
        xFr = np.array([])
        yFr = []
        poly = Polygon(list(zip(xC, yC)))
        for xi in (x):
            for yi in (y):
                if (Point([xi,yi]).within(poly)):
                    xFr = np.append(xFr,xi)
                    yFr.append(yi)

        return xC, yC, xFr, yFr
#-------------------------------------------------------#
# Function definitions
def rotate(pt, origin, angleD):
    angle = angleD*np.pi/180
    ox, oy = origin
    px, py = pt

    qx = ox + math.cos(angle) * (px - ox) - math.sin(angle) * (py - oy)
    qy = oy + math.sin(angle) * (px - ox) + math.cos(angle) * (py - oy)
    return qx, qy 

#-------------------------------------------------------#    
## Create the twin turbines
# Downstream unit
turbLoc = [12.5, 2.5]
elev = 0.0 
slope = 0.0
orientation = 90
test = TwinEM0(turbLoc,elev,slope,orientation)

## Get the bathymetry data
# Foundation
xF, yF = test.getBathyBase()

# 2 corner columns and symmetry
xC, yC = test.getBathyCols()
xC = np.concatenate((xC,-xC))
yC = np.concatenate((yC,yC))
xC += turbLoc[0]
yC += turbLoc[1]
        
# Frame and symmetry
xPoly, yPoly, xFr, yFr = test.getBathyFrame()
xPoly = np.concatenate((xPoly,-xPoly))
yPoly = np.concatenate((yPoly,yPoly))
xPoly += turbLoc[0]
yPoly += turbLoc[1]

xFr = np.concatenate((xFr,-xFr))
yFr = np.concatenate((yFr,yFr))
yFr = np.array(yFr)
xFr += turbLoc[0]
yFr += turbLoc[1]

# Rotate all turbine points to align with the channel
for i in range(len(xF)):
    xF[i], yF[i] = rotate([xF[i],yF[i]],turbLoc,orientation)

for i in range(len(xC)):
    xC[i], yC[i] = rotate([xC[i],yC[i]],turbLoc,orientation)

for i in range(len(xFr)):
    xFr[i], yFr[i] = rotate([xFr[i],yFr[i]],turbLoc,orientation)

for i in range(len(xPoly)):
    xPoly[i], yPoly[i] = rotate([xPoly[i],yPoly[i]],turbLoc,orientation)

# Write to file
with open("bathymetry_test.xyz","w") as file:
    for i in range(len(xF)):
        file.write('\t' + str(xF[i]) + '\t' + str(yF[i]) + '\t' + str(test.fndH + elev) + '\n')

    for i in range(len(xC)):
        file.write('\t' + str(xC[i]) + '\t' + str(yC[i]) + '\t' + str(test.colH + elev) + '\n')

    for i in range(len(xFr)):
        file.write('\t' + str(xFr[i]) + '\t' + str(yFr[i]) + '\t' + str(test.colH + elev) + '\n')

# %%
plt.plot(xF,yF,'x')
plt.plot(xC,yC,'o')
plt.plot(xFr,yFr,'*')

plt.grid()
plt.show()

















